﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;             //【Debug】
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【BitmapData】
using System.Runtime.InteropServices; //【Marshal.Copy】【Win32.API】
using System.Threading;               //【Threadを使う為に必要】
using Microsoft.Kinect;               //【参照設定】プロジェクト(P)⇒参照の追加(F)⇒.NETタブ⇒Microsoft.Kinectを選択
using Microsoft.Speech.AudioFormat;   //【参照設定】C:\Program Files\Microsoft SDKs\Kinect\v1.0\Assemblies\Microsoft.Kinect.dll
using Microsoft.Speech.Recognition;   //【参照設定】C:\Program Files\Microsoft SDKs\Speech\v11.0\Assembly\Microsoft.Speech.dll
using Microsoft.Speech.Synthesis;     //【音声合成】 


namespace KinectSpeechControl
{
    public partial class Form1 : Form
    {
        //【KINECT関連】--------------------------------------------------------------------
        private KinectSensor KS_KinectSensor;   //【KINECTセンサ】

        //【音声認識関連】
        private KinectAudioSource KS_Source;    //【KINECTオーディオ入力】
        private RecognizerInfo KS_RI;           //【音声認識情報】
        private SpeechRecognitionEngine KS_SRE; //【音声認識エンジン】
        private Choices KS_SelectiveWords;      //【音声認識候補の語】
        private GrammarBuilder KS_GB;           //【文法生成】
        private Grammar KS_G;                   //【文法】
        private string[] KS_Choice = { "red", "yellow", "green", "blue", "start", "stop", 
                                       "right", "left", "up","front", "down", "exit",
                                       "measure", "sensor", "audio", "sound", "hello", "Japan", "aircon" };
        private string KS_RecognizedWord="";
        private string KS_PastWord = "";
        private float KS_Confidence;
        private Boolean KS_SREflag = false;

        private SpeechSynthesizer KS_SS; //【音声合成】
        

        //【カラー画像キャプチャー関係】
        private ColorImageFrame KS_ColorFrame;  //【KINECTのColorImageFrame】
        private byte[] KS_ColorImageDataXRGB;   //【KINECTのColorImageData】32bit/画素,X8R8G8B8表示形式
        private int KS_ColorFrameWidth;         //【KINECTのColorFrameの横方向画素数】
        private int KS_ColorFrameHeight;        //【KINECTのColorFrameの縦方向画素数】
        private Bitmap KS_bmp1;                 //【ColorImage表示用Bitmapオブジェクト】
        private BitmapData KS_data1;            //【ColorImage用BitmapData】
        private Rectangle KS_rect1;             //【ColorImage表示枠】
        private bool KS_flagFlip = true;        //【左右反転フラグ】

        //【USB ADC基板のコントロール】
        private String COMPORT; //【A/Dコンバータ通信用ComPort番号】←ここはデバイス検出状況に応じて変更してください。
        System.IO.Ports.SerialPort SerialPort_for_ADC = new System.IO.Ports.SerialPort(); //【ADC用シリアルポート】
        private string RxMsg; //【受信文字列】
        private Boolean NTL_SREflag = false;

        private string NTL_bmpFileName = "../../image/Display.bmp";
        private int ADcount = 0;
        private float SensorValue;
        private string sSensorValue = "";
        private int MeasureValue = 0;
        private float[] FIFO = new float[640];
        private int wPtr = 0;
        private int rPtr = 0;
        private Boolean flag = true;
        private int NTL_Ch = 0;
        private int NTL_pCh = 0;


        public Form1()
        {
            InitializeComponent();
            //【利用可能なKINECTセンサを取得する】
            KS_KinectSensor = (from sensorToCheck
                               in KinectSensor.KinectSensors
                               where sensorToCheck.Status == KinectStatus.Connected
                               select sensorToCheck).FirstOrDefault();
            if (KS_KinectSensor == null)
            {
                pictureBox2.Image = imageList1.Images[1];
                MessageBox.Show(
                   "接続可能なKINECTセンサが見当たりません。",
                   "KINECT for Windows Error Message",
                   MessageBoxButtons.OK
                );
                return;
            }
            else
            {
                pictureBox2.Image = imageList1.Images[0];
            }

            //【KINECTの設定】ColorStreamを許可*******************************************************************
            KS_KinectSensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30); //【ColorStream許可】
            KS_KinectSensor.Start(); //【KINECTスタート】

            //【KINECTデータ取得イベントの設定とデータバッファの確保】********************************************
            KS_KinectSensor.ColorFrameReady += ColorImageReady; //【準備完了でColorImageReadyがcallされる】
            KS_ColorImageDataXRGB = new byte[KS_KinectSensor.ColorStream.FramePixelDataLength];      //【カラー画像データ用】
            KS_ColorFrameWidth = KS_KinectSensor.ColorStream.FrameWidth;   //【ColorImageの横方向画素数】
            KS_ColorFrameHeight = KS_KinectSensor.ColorStream.FrameHeight; //【ColorImageの縦方向画素数】
            KS_KinectSensor.ElevationAngle = 0; //【KINECTセンサの上下角を0°に設定】設定範囲±27°
            KS_bmp1 = new Bitmap(KS_ColorFrameWidth, KS_ColorFrameHeight, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【ColorImage表示用】
            KS_rect1 = new Rectangle(0, 0, KS_ColorFrameWidth, KS_ColorFrameHeight); //【ColorImage用枠】

            KS_Source = KS_KinectSensor.AudioSource;
            KS_Source.EchoCancellationMode = EchoCancellationMode.None; //【オーディオ・エコーキャンセラーを使わない】
            KS_Source.AutomaticGainControlEnabled = false; //【音声認識の為に自動ゲイン・コントロールをオフにする】
            KS_Source.SoundSourceAngleChanged +=new EventHandler<SoundSourceAngleChangedEventArgs>(SoundSourceAngleChanged);
            KS_Source.BeamAngleChanged +=new EventHandler<BeamAngleChangedEventArgs>(BeamAngleChanged);

            KS_RI = GetKinectRecognizer();
            if (KS_RI == null)
            {
                pictureBox2.Image = imageList1.Images[1];
                MessageBox.Show(
                    "KINECT音声認識が見つかりません。KINECTの接続をご確認ください。",
                    "KINECT for Windows Error Message",
                    MessageBoxButtons.OK
                );
                return;
            }
            label2.Text = KS_RI.Name; //【取得した音声認識名称を表示する】

            int wait = 4; //デバイスが正常に動作するように初期化後４秒間待つ必要がある。
            while (wait > 0)
            {
                label5.Text = "デバイスが音声認識の準備を完了するまであと" + wait.ToString() + "秒です。";
                wait--;
                Thread.Sleep(1000);
            }

            KS_SRE = new SpeechRecognitionEngine(KS_RI.Id);
            KS_SelectiveWords = new Choices();
            for (int i = 0; i < KS_Choice.Length; i++) { KS_SelectiveWords.Add(KS_Choice[i]); }
            
            KS_GB = new GrammarBuilder();
            KS_GB.Culture = KS_RI.Culture; // Specify the culture to match the recognizer in case we are running in a different culture.                                 
            KS_GB.Append(KS_SelectiveWords); 
            KS_G = new Grammar(KS_GB); //文法のインスタンスを作成する
            KS_SRE.LoadGrammar(KS_G);  //音声認識に文法インスタンスをロードする
            KS_SRE.SpeechRecognized += SreSpeechRecognized; //音声認識処理実行イベントのcall先をSreSpeechRecognizedに設定する
            Stream s=KS_Source.Start();
            KS_SRE.SetInputToAudioStream(s, new SpeechAudioFormatInfo(EncodingFormat.Pcm, 16000, 16, 1, 32000, 2, null));
            label5.Text = "こんにちは。音声認識を行います。登録された言葉を発音してみてください。";
            KS_SRE.RecognizeAsync(RecognizeMode.Multiple);

            KS_SS = new SpeechSynthesizer();
            KS_SS.SetOutputToDefaultAudioDevice();
            //【備考】何故か私の環境では日本語で話すようだ。
            //     KS_SS.Speak("Hello. Speech Recognition is started. Please say the reserved word.");
            KS_SS.Speak("こんにちは。音声認識を行います。登録された言葉を発音してみてください。");

//            pictureBox4.Image = Bitmap.FromFile(NTL_bmpFileName);
            timer1.Interval = 100;
            timer1.Enabled = true;
            for (int i = 0; i < FIFO.Length; i++) { FIFO[i] = 0; }
            string[] pn = System.IO.Ports.SerialPort.GetPortNames();
            if (pn.Length == 0)
            {
                KS_SS.Speak("残念ながら、接続可能なシリアルポートが見つかりません。");
                return;
            }
            COMPORT = pn[0]; //【一つしかシリアルポートが接続されていない前提で選択】
            
            InitializeSerialPort(COMPORT);
            ADCInitialize();
            SerialPort_for_ADC.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(SerialPort_for_ADC_DataReceived);
            ADCConvert();  //【次のA/D変換データを要求する】
            radioButton1.CheckedChanged += new EventHandler(radioButton1_CheckedChanged);
            radioButton2.CheckedChanged += new EventHandler(radioButton2_CheckedChanged);
            radioButton3.CheckedChanged += new EventHandler(radioButton3_CheckedChanged);
            radioButton4.CheckedChanged += new EventHandler(radioButton4_CheckedChanged);

        }

        //***************************************************************
        //【RadioButtonが変化した場合に発生するイベント】
        //***************************************************************
        void radioButton1_CheckedChanged(object sender, EventArgs e) { NTL_Ch = 0; }
        void radioButton2_CheckedChanged(object sender, EventArgs e) { NTL_Ch = 1; }
        void radioButton3_CheckedChanged(object sender, EventArgs e) { NTL_Ch = 2; }
        void radioButton4_CheckedChanged(object sender, EventArgs e) { NTL_Ch = 3; }


        //***************************************************************
        //【ColorImageの取得イベント】
        //***************************************************************
        private void ColorImageReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (KS_ColorFrame = e.OpenColorImageFrame())
            {
                if (KS_ColorFrame != null)
                {
                    KS_ColorFrame.CopyPixelDataTo(KS_ColorImageDataXRGB); //【KINECTのColor画像を配列PixelDataにコピー】
                    KS_data1 = KS_bmp1.LockBits(KS_rect1, ImageLockMode.WriteOnly, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【Lock】
                    Marshal.Copy(KS_ColorImageDataXRGB, 0, KS_data1.Scan0, KS_ColorImageDataXRGB.Length); //【UnmanagedMemoryのBitmapDataに配列PixelDataをコピー】
                    KS_bmp1.UnlockBits(KS_data1);           //【Lock解除】
                    if (KS_flagFlip) { KS_bmp1.RotateFlip(RotateFlipType.Rotate180FlipY); } //【左右反転】
                    pictureBox1.Image = KS_bmp1;            //【pictureBox1に画像表示】
                    //             Application.DoEvents();

                    this.label22.Text = sSensorValue;
                    FIFO[wPtr] = SensorValue; //【範囲】0.0～1.0
                    rPtr = wPtr;
                    wPtr++; if (wPtr == FIFO.Length) { wPtr = 0; }
           //         pictureBox4.Image = Bitmap.FromFile(NTL_bmpFileName);
                    Graphics g = Graphics.FromImage(pictureBox1.Image);
                    SolidBrush newbrush = new SolidBrush(Color.FromArgb(70, Color.Red));
                    int i = 0;
                    PointF[] p1 = new PointF[FIFO.Length];
                    for (i = 0; i < FIFO.Length; i++)
                    {
                        p1[i] = new PointF((float)i, 300.0f * (1.0f - (float)FIFO[rPtr])+100.0f);
                        rPtr--; if (rPtr < 0) { rPtr = FIFO.Length - 1; }
                    }
                    g.DrawLines(new Pen(Color.Red, 3.0f), p1);
                    g.Dispose();
                }
                else
                {
                    //【ImageFrameがnullだったのは要求が間に合わなかったと考えられます】
                }

                if (KS_SREflag)
                {
                    KS_SREflag = false;
                    label5.Text = "【音声認識された語】" + KS_RecognizedWord + "  【確からしさ】" + KS_Confidence.ToString();
                    switch (KS_RecognizedWord)
                    {
                        case "red":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Red.jpg");
                            break;
                        case "yellow":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Yellow.jpg");
                            break;
                        case "green":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Green.jpg");
                            break;
                        case "blue":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Blue.jpg");
                            break;
                        case "start":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Start.jpg"); break;
                        case "stop":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Stop.jpg"); break;
                        case "right":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Right.jpg"); break;
                        case "left":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Left.jpg"); break;
                        case "up":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/UpDir.jpg");
                            //                     KS_SS.Speak("キネクトのヘッドを上に向けます");
                            if (KS_PastWord != KS_RecognizedWord) { KS_KinectSensor.ElevationAngle = 27; textBox1.Text = "27"; }
                            break;
                        case "front":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/FrontDir.jpg");
                            //                     KS_SS.Speak("キネクトのヘッドを上に向けます");
                            if (KS_PastWord != KS_RecognizedWord) { KS_KinectSensor.ElevationAngle = 0; textBox1.Text = "27"; }
                            break;
                        case "down":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/DownDir.jpg");
                            //                        KS_SS.Speak("キネクトのヘッドを下に向けます");
                            if (KS_PastWord != KS_RecognizedWord) { KS_KinectSensor.ElevationAngle = -27; textBox1.Text = "-27"; }
                            break;
                        case "exit":
                            //                        KS_SS.Speak("終了します");
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Exit.jpg"); break;
                        case "measure":
                            //                        KS_SS.Speak("計測します");
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Measure.jpg"); break;
                        case "sensor":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Sensor.jpg"); break;
                        case "audio":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Audio.jpg"); break;
                        case "sound":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Sound.jpg"); break;
                        case "hello":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Hello.jpg"); break;
                        case "Japan":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/Japan.jpg"); break;
                        case "aircon":
                            pictureBox3.Image = System.Drawing.Image.FromFile("../../image/AirCon.jpg"); break;
                    }
                    KS_PastWord = KS_RecognizedWord;
                }
            }
        }

        //***************************************************************
        //【終了要求ボタン】
        //***************************************************************
        private void button1_Click(object sender, EventArgs e)
        {
            if (KS_KinectSensor != null)
            {
                KS_KinectSensor.Stop();    //【KINECTセンサを終了】★これを実行しないとレーザーが消えない
                KS_KinectSensor.Dispose(); //【KINECTセンサのオブジェクトを解放する】
                KS_SRE.RecognizeAsyncStop();
                KS_Source.Stop();
            }
            this.Close();              //【Formを閉じる】
        }

        //***************************************************************
        //【上下角変更ボタン】KINECTの上下角を変更。±27°の範囲内。時間を置かないとエラーが発生する
        //***************************************************************
        private void button2_Click(object sender, EventArgs e)
        {
            Int32 angle = int.Parse(textBox1.Text); //【設定角度を読み込む】±27°
            if (angle <= KS_KinectSensor.MaxElevationAngle && angle >= KS_KinectSensor.MinElevationAngle)
            {
                KS_KinectSensor.ElevationAngle = angle; //【上下角の変更】
            }
        }

        //***************************************************************
        //【カラー画像の左右反転指定フラグ】
        //***************************************************************
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked) { KS_flagFlip = true; } else { KS_flagFlip = false; }
        }

        //***************************************************************
        //【GetKinectRecognizer】
        //***************************************************************
        private static RecognizerInfo GetKinectRecognizer()
        {
            Func<RecognizerInfo, bool> matchingFunc = r =>
            {
                string value;
                r.AdditionalInfo.TryGetValue("Kinect", out value);
                return "True".Equals(value, StringComparison.InvariantCultureIgnoreCase)
                    && "en-US".Equals(r.Culture.Name, StringComparison.InvariantCultureIgnoreCase);
              //      && "ja-JP".Equals(r.Culture.Name, StringComparison.InvariantCultureIgnoreCase);
            };
            return SpeechRecognitionEngine.InstalledRecognizers().Where(matchingFunc).FirstOrDefault();
        }

        //***************************************************************
        //【SreSpeechRecognized】 
        //***************************************************************
        private void SreSpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            KS_Confidence = e.Result.Confidence;
            if (e.Result.Confidence >= 0.7)
            {
                KS_RecognizedWord = e.Result.Text;
                KS_SREflag = true;
                NTL_SREflag = true;
            }
        }

        //***************************************************************
        //【SreSpeechRecognitionRejected】static 
        //***************************************************************
        private void SreSpeechRecognitionRejected(object sender, SpeechRecognitionRejectedEventArgs e)
        {
            Console.WriteLine("\nSpeech Rejected");
            if (e.Result != null)
            {
                DumpRecordedAudio(e.Result.Audio);
            }
        }

        //***************************************************************
        //【SreSpeechHypothesized】static 
        //***************************************************************
        private void SreSpeechHypothesized(object sender, SpeechHypothesizedEventArgs e)
        {
            Console.Write("\rSpeech Hypothesized: \t{0}", e.Result.Text);
        }

        //***************************************************************
        //【DumpRecordedAudio】
        //***************************************************************
        private static void DumpRecordedAudio(RecognizedAudio audio)
        {
            if (audio == null) { return; }

            int fileId = 0;
            string filename;
            while (File.Exists((filename = "RetainedAudio_" + fileId + ".wav"))) { fileId++; } //【】

            Console.WriteLine("\nWriting file: {0}", filename);
            using (var file = new FileStream(filename, System.IO.FileMode.CreateNew))
            {
                audio.WriteToWaveStream(file);
            }
        }

        //***************************************************************
        //【SoundSourceAngleChangedの取得イベント】
        //***************************************************************
        private void SoundSourceAngleChanged(object sender, SoundSourceAngleChangedEventArgs e)
        {
            label3.Text = "【音源の角度】" + e.Angle.ToString() + "[度]";
        }

        //***************************************************************
        //【BeamAngleChangedの取得イベント】
        //***************************************************************
        private void BeamAngleChanged(object sender, BeamAngleChangedEventArgs e)
        {
            label4.Text = "【ビームの角度】" + e.Angle.ToString() + "[度]";
        }

        //***********************************************************************
        // 時間待ち(ミリ秒単位)
        //***********************************************************************
        [DllImport("kernel32.dll")]
        public static extern void Sleep(Int32 dwMilliseconds);

        public Boolean InitializeSerialPort(string com)
        {
            //【ADC用シリアルポートの設定】
            this.SerialPort_for_ADC.PortName = COMPORT;
            // トラ技増刊号の付録基板は疑似シリアルポート
            //     this.SerialPort_for_ADC.BaudRate = 115200;
            //     this.SerialPort_for_ADC.DataBits = 8;
            //     this.SerialPort_for_ADC.StopBits = System.IO.Ports.StopBits.One;
            //     this.SerialPort_for_ADC.Parity = System.IO.Ports.Parity.None;
            //     this.SerialPort_for_ADC.ReadTimeout = 50;
            //     this.SerialPort_for_ADC.ReadBufferSize = 256;
            //     this.SerialPort_for_ADC.WriteBufferSize = 256;
            try { this.SerialPort_for_ADC.Open(); }
            catch (Exception) { MessageBox.Show("シリアルポートが開けません"); return false; }
            return true;
        }

        void SerialPort_for_ADC_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            if (flag)
            {
                ADCGetValue(); //【シリアルポートからデータを読み込む】
                if (NTL_Ch != NTL_pCh)
                {
                //    Sleep(100);
                    //   this.SerialPort_for_ADC.DiscardInBuffer();    //【バッファ内データを破棄する】
                    ADCInitialize();
                    NTL_pCh = NTL_Ch;
                //    Sleep(100);
                }
                if (NTL_SREflag)
                {
                    NTL_SREflag = false;
                    switch (KS_RecognizedWord)
                    {
                        case "red":
                            this.SerialPort_for_ADC.Write("PW08\r\n");  //【送信】bit3
                            Sleep(50);
                            RxMsg = this.SerialPort_for_ADC.ReadLine();
                            break;
                        case "yellow":
                            this.SerialPort_for_ADC.Write("PW10\r\n");  //【送信】bit4
                            Sleep(50);
                            RxMsg = this.SerialPort_for_ADC.ReadLine();
                            break;
                        case "green":
                            this.SerialPort_for_ADC.Write("PW40\r\n");  //【送信】bit6
                            Sleep(50);
                            RxMsg = this.SerialPort_for_ADC.ReadLine();
                            break;
                        case "blue":
                            this.SerialPort_for_ADC.Write("PW20\r\n");  //【送信】bit5
                            Sleep(50);
                            RxMsg = this.SerialPort_for_ADC.ReadLine();
                            break;
                        case "aircon":
                            this.SerialPort_for_ADC.Write("PW01\r\n");  //【送信】bit0
                            Sleep(50);
                            RxMsg = this.SerialPort_for_ADC.ReadLine();
                            Sleep(500);
                            this.SerialPort_for_ADC.Write("PW00\r\n");  //【送信】bit0
                            Sleep(50);
                            RxMsg = this.SerialPort_for_ADC.ReadLine();
                            break;
                        default:
                            this.SerialPort_for_ADC.Write("PW00\r\n");  //【送信】
                            RxMsg = this.SerialPort_for_ADC.ReadLine();
                            break;
                    }
                }
                ADCConvert();  //【次のA/D変換データを要求する】

            }
        }

        public void TerminateSerialPort()
        {
            this.SerialPort_for_ADC.Close();
        }


        // ■■■■■■■■■■　【A/Dコンバータの初期化処理】　■■■■■■■■■■
        public void ADCInitialize()
        {
            //【接続確認】特に値のチェックなどはしていません。
            this.SerialPort_for_ADC.Write("E\r\n");       //【送信】"E"+CR+LF
            Sleep(50);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            //【ADCモードレジスタ書込み】000A　ADC変換レートを(16.7Hzに)設定する
            // this.SerialPort_for_ADC.Write("C080001\r\n"); //【送信】"C08"+"0001"+CR+LF 500Hz
            // this.SerialPort_for_ADC.Write("C080002\r\n"); //【送信】"C08"+"0002"+CR+LF 250Hz
            // this.SerialPort_for_ADC.Write("C080003\r\n"); //【送信】"C08"+"0003"+CR+LF 125Hz
            // this.SerialPort_for_ADC.Write("C080004\r\n"); //【送信】"C08"+"0004"+CR+LF 62.5Hz
            this.SerialPort_for_ADC.Write("C080005\r\n"); //【送信】"C08"+"0005"+CR+LF 50Hz
            // this.SerialPort_for_ADC.Write("C080006\r\n"); //【送信】"C08"+"0006"+CR+LF 39.2Hz
            // this.SerialPort_for_ADC.Write("C080007\r\n"); //【送信】"C08"+"0007"+CR+LF 33.3Hz
            // this.SerialPort_for_ADC.Write("C080008\r\n"); //【送信】"C08"+"0008"+CR+LF 19.6Hz
            // this.SerialPort_for_ADC.Write("C08000A\r\n"); //【送信】"C08"+"000A"+CR+LF 16.7Hz
            Sleep(50);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            //【ADC設定レジスタ書込み】5080 【コマンド】C105080 + CR + LF
            if (radioButton1.Checked) { this.SerialPort_for_ADC.Write("C105080\r\n"); NTL_Ch = 0; } //【入力】チャンネル0
            if (radioButton2.Checked) { this.SerialPort_for_ADC.Write("C105081\r\n"); NTL_Ch = 1; } //【入力】チャンネル1
            if (radioButton3.Checked) { this.SerialPort_for_ADC.Write("C105082\r\n"); NTL_Ch = 2; } //【入力】チャンネル2
            if (radioButton4.Checked) { this.SerialPort_for_ADC.Write("C105086\r\n"); NTL_Ch = 3; } //【入力】内蔵温度センサ
            NTL_pCh = NTL_Ch;
            Sleep(50);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            //【IOレジスタ】0A　【コマンド】C280A + CR + LF
            this.SerialPort_for_ADC.Write("C280A\r\n");   //【送信】電流源は未使用
            Sleep(50);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            //【ADCオフセットレジスタ】800000 【コマンド】C30800000 + CR + LF
            this.SerialPort_for_ADC.Write("C30800000\r\n"); //【送信】
            Sleep(50);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            this.SerialPort_for_ADC.Write("PM80\r\n"); //【送信】
            Sleep(50);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            this.SerialPort_for_ADC.Write("PW78\r\n"); //【送信】
            Sleep(3000);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            this.SerialPort_for_ADC.Write("PW00\r\n"); //【送信】
            Sleep(50);                                    //【待機】50m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
            this.SerialPort_for_ADC.DiscardInBuffer();    //【バッファ内データを破棄する】
        }

        // ■■■■■■■■■■　【A/Dコンバータの終了処理】　■■■■■■■■■■
        public void ADCTerminate()
        {
            this.SerialPort_for_ADC.Write("C08600A\r\n"); //【送信】パワーダウンモード
            Sleep(5);                                     //【待機】5m秒
            RxMsg = this.SerialPort_for_ADC.ReadLine();   //【受信】
        }

        // ■■■■■■■■■■　【A/D変換処理】　■■■■■■■■■■
        // ここにA/D変換読み取り処理を挿入する。設定範囲を0≦MeasureValue≦255とする。
        void ADCConvert()
        {
            this.SerialPort_for_ADC.Write("R\r\n");     //【送信】AD変換結果を取得する
        }

        void ADCGetValue()
        {
            ADcount = 0;
            try { RxMsg = this.SerialPort_for_ADC.ReadLine(); } //【受信】
            catch { RxMsg = "000000"; }
            if (RxMsg.Substring(RxMsg.Length - 1, 1) == "\n") { RxMsg = RxMsg.Substring(0, RxMsg.Length - 2); }
            if (RxMsg.Substring(RxMsg.Length - 1, 1) == "\r") { RxMsg = RxMsg.Substring(0, RxMsg.Length - 1); }
            if (RxMsg.Substring(0, 1) == "\0") { RxMsg = RxMsg.Substring(1, RxMsg.Length - 1); }
            ADcount = Convert.ToInt32(RxMsg, 16); //★Convertクラスを使って16進数を符号付き整数として変換する
            SensorValue = 1.17f * (float)ADcount / 16777216.0f; //内蔵温度計の値を温度に変換
            if (NTL_Ch == 3)
            {
                float Temperature;
                Temperature = 1.17f * (float)ADcount / (16777216.0f * 0.00081f) - 273.15f;
                sSensorValue = Temperature.ToString() + "[℃]";
                SensorValue = (Temperature - 10.0f) * 0.025f;
            }
            else
            {
                sSensorValue = SensorValue.ToString() + "[V]";
            }
            MeasureValue = (int)(SensorValue * 255.0f);
            if (MeasureValue < 0) { MeasureValue = 0; }
            if (MeasureValue > 255) { MeasureValue = 255; }

        }


    }
}
